@@ -73,6 +73,8 @@ static void       print_collf(FILE *collf, struct header *hp);
 /* Write a file, ex-like if f set */
 static int        exwrite(char const *name, FILE *fp, int f);
 
+/* Parse off the message header from fp and store relevant fields in hp,
+ * replace _coll_fp with a shiny new version without any header */
 static enum okay  makeheader(FILE *fp, struct header *hp, si8_t *checkaddr_err);
 
 /* Edit the message being collected on fp.  On return, make the edit file the
@@ -228,7 +230,7 @@ print_collf(FILE *cf, struct header *hp)
 {
    char *lbuf = NULL; /* TODO line pool */
    sighandler_type sigint;
-   FILE *volatile obuf = stdout;
+   FILE * volatile obuf = stdout;
    struct attachment *ap;
    char const *cp;
    enum gfield gf;
@@ -609,16 +611,21 @@ collect(struct header *hp, int printheaders, struct message *mp,
    struct ignoretab *quoteig;
    int lc, cc, c, t;
    int volatile escape, getfields;
-   char *linebuf = NULL, *quote = NULL;
+   char *linebuf, *quote;
    char const *cp;
-   size_t linesize = 0; /* TODO line pool */
+   size_t i, linesize; /* TODO line pool */
    long cnt;
    enum sendaction action;
    sigset_t oset, nset;
    sighandler_type savedtop;
+   FILE * volatile sigfp;
    NYD_ENTER;
 
    _coll_fp = NULL;
+   sigfp = NULL;
+   linesize = 0;
+   linebuf = quote = NULL;
+
    /* Start catching signals from here, but we're still die on interrupts
     * until we're in the main loop */
    sigemptyset(&nset);
@@ -757,12 +764,10 @@ jcont:
    /* No tilde escapes, interrupts not expected.  Simply copy STDIN */
    if (!(options & (OPT_INTERACTIVE | OPT_t_FLAG | OPT_TILDE_FLAG))) {
       linebuf = srealloc(linebuf, linesize = LINESIZE);
-      while ((cnt = fread(linebuf, sizeof *linebuf, linesize, stdin)) > 0) {
-         if ((size_t)cnt != fwrite(linebuf, sizeof *linebuf, cnt, _coll_fp))
+      while ((i = fread(linebuf, sizeof *linebuf, linesize, stdin)) > 0) {
+         if (i != fwrite(linebuf, sizeof *linebuf, i, _coll_fp))
             goto jerr;
       }
-      if (fflush(_coll_fp))
-         goto jerr;
       goto jout;
    }
 
@@ -783,11 +788,10 @@ jcont:
             pstate |= PS_t_FLAG;
             if (makeheader(_coll_fp, hp, checkaddr_err) != OKAY)
                goto jerr;
-            rewind(_coll_fp);
             options &= ~OPT_t_FLAG;
             continue;
          } else if ((options & OPT_INTERACTIVE) && ok_blook(ignoreeof)) {
-            printf(_("*ignoreeof* set, use \".\" to terminate letter\n"));
+            printf(_("*ignoreeof* set, use \"~.\" to terminate letter\n"));
             continue;
          }
          break;
@@ -811,7 +815,8 @@ jputline:
       if (linebuf[0] != escape)
          goto jputline;
 
-      tty_addhist(linebuf, TRU1);
+      if (!(options & OPT_t_FLAG))
+         tty_addhist(linebuf, TRU1);
 
       c = linebuf[1];
       switch (c) {
@@ -996,48 +1001,91 @@ jputline:
          /* Last the lengthy help string.  (Very ugly, but take care for
           * compiler supported string lengths :() */
          puts(_(
-"-------------------- ~ ESCAPES ----------------------------\n"
-"~~             Quote a single tilde\n"
-"~@ [file ...]  Edit attachment list\n"
-"~b users       Add users to \"blind\" Bcc: list\n"
-"~c users       Add users to Cc: list\n"
-"~d             Read in dead.letter\n"
-"~e             Edit the message buffer\n"
-"~F messages    Read in messages including all headers, don't indent lines\n"
-"~f messages    Like ~F, but honour the `ignore' / `retain' configuration\n"
-"~h             Prompt for Subject:, To:, Cc: and \"blind\" Bcc:\n"));
+"TILDE ESCAPES (to be placed after a newline) excerpt:\n"
+"~.            Commit and send message\n"
+"~: <command>  Execute a mail command\n"
+"~<! <command> Insert output of command\n"
+"~@ [<files>]  Edit attachment list\n"
+"~A            Insert *Sign* variable (`~a' inserts *sign*)\n"
+"~c <users>    Add users to Cc: list (`~b' for Bcc:)\n"
+"~d            Read in *DEAD* (dead.letter)\n"
+"~e            Edit message via *EDITOR*"
+         ));
          puts(_(
-"~R file        Read in a file, indent lines\n"
-"~r file        Read in a file\n"
-"~p             Print the message buffer\n"
-"~q             Abort message composition and save text to DEAD\n"
-"~M messages    Read in messages, keep all header lines, indent lines\n"
-"~m messages    Like ~M, but honour the `ignore' / `retain' configuration\n"
-"~s subject     Set Subject:\n"
-"~t users       Add users to To: list\n"));
+"~F <msglist>  Read in with headers, don't *indentprefix* lines\n"
+"~f <msglist>  Like ~F, but honour `ignore' / `retain' configuration\n"
+"~H            Edit From:, Reply-To: and Sender:\n"
+"~h            Prompt for Subject:, To:, Cc: and \"blind\" Bcc:\n"
+"~i <variable> Insert a value and a newline\n"
+"~M <msglist>  Read in with headers, *indentprefix* (`~m': `retain' etc.)\n"
+"~p            Print current message compose buffer\n"
+"~r <file>     Read in a file (`~R' *indentprefix* lines)"
+         ));
          puts(_(
-"~U messages    Read in message(s) without any headers, indent lines\n"
-"~u messages    Read in message(s) without any headers\n"
-"~v             Invoke alternate editor ($VISUAL) on message\n"
-"~w file        Write message onto file\n"
-"~x             Abort message composition and discard message\n"
-"~!command      Invoke the shell\n"
-"~:command      Execute a regular command\n"
-"-----------------------------------------------------------\n"));
+"~s <subject>  Set Subject:\n"
+"~t <users>    Add users to To: list\n"
+"~u <msglist>  Read in message(s) without headers (`~U' indents lines)\n"
+"~v            Edit message via *VISUAL*\n"
+"~w <file>     Write message onto file\n"
+"~x            Abort composition, discard message (`~q' saves in *DEAD*)\n"
+"~| <command>  Pipe message through shell filter"
+         ));
          break;
       }
    }
 
 jout:
-   if (_coll_fp != NULL) {
-      if ((cp = ok_vlook(NAIL_TAIL)) != NULL) {
-         if (putesc(cp, _coll_fp) < 0)
-            goto jerr;
-         if ((options & OPT_INTERACTIVE) && putesc(cp, stdout) < 0)
+   /* Place signature? */
+   if((cp = ok_vlook(signature)) != NULL && *cp != '\0'){
+      if((quote = file_expand(cp)) == NULL){
+         n_err(_("*signature* expands to invalid file: \"%s\"\n"), cp);
+         goto jerr;
+      }
+      if((sigfp = Fopen(cp = quote, "r")) == NULL){
+         n_err(_("Can't open *signature* \"%s\": %s\n"), cp, strerror(errno));
+         goto jerr;
+      }
+
+      if(linebuf == NULL)
+         linebuf = smalloc(linesize = LINESIZE);
+      c = '\0';
+      while((i = fread(linebuf, sizeof *linebuf, linesize, UNVOLATILE(sigfp)))
+            > 0){
+         c = linebuf[i - 1];
+         if(i != fwrite(linebuf, sizeof *linebuf, i, _coll_fp))
             goto jerr;
       }
-      rewind(_coll_fp);
+
+      /* C99 */{
+         FILE *x = UNVOLATILE(sigfp);
+         int e = errno, ise = ferror(x);
+
+         sigfp = NULL;
+         Fclose(x);
+
+         if(ise){
+            n_err(_("Errors while reading *signature* \"%s\": %s\n"),
+               cp, strerror(e));
+            goto jerr;
+         }
+      }
+
+      if(c != '\0' && c != '\n')
+         putc('\n', _coll_fp);
    }
+
+   if(fflush(_coll_fp))
+      goto jerr;
+
+   if ((cp = ok_vlook(NAIL_TAIL)) != NULL) {
+      if (putesc(cp, _coll_fp) < 0)
+         goto jerr;
+      if ((options & OPT_INTERACTIVE) && putesc(cp, stdout) < 0)
+         goto jerr;
+   }
+   rewind(_coll_fp);
+
+jleave:
    if (linebuf != NULL)
       free(linebuf);
    handlerpop();
@@ -1056,11 +1104,13 @@ jout:
    return _coll_fp;
 
 jerr:
+   if(sigfp != NULL)
+      Fclose(UNVOLATILE(sigfp));
    if (_coll_fp != NULL) {
       Fclose(_coll_fp);
       _coll_fp = NULL;
    }
-   goto jout;
+   goto jleave;
 }
 
 FL void
