@@ -142,27 +142,32 @@ static bool_t     _source_file(char const *file, bool_t silent_error);
 static void
 _findmail(char *buf, size_t bufsize, char const *user, bool_t force)
 {
-   char *cp;
+   char const *ibox, *cp;
    NYD_ENTER;
 
-   if (!strcmp(user, myname) && !force && (cp = ok_vlook(folder)) != NULL) {
-      switch (which_protocol(cp)) {
-      case PROTO_IMAP:
-         if (strcmp(cp, protbase(cp)))
-            goto jcopy;
-         snprintf(buf, bufsize, "%s/INBOX", cp);
-         goto jleave;
-      default:
-         break;
-      }
+   ibox = ok_vlook(inbox);
+
+   /* Heirloom compatibility: an IMAP *folder* becomes "%" */
+   if (!force && ibox == NULL && !strcmp(user, myname) &&
+         (cp = ok_vlook(folder)) != NULL && which_protocol(cp) == PROTO_IMAP) {
+      OBSOLETE("no more expansion of *folder* in \"%\": please set *inbox*");
+      goto jcopy;
    }
 
-   if (force || (cp = ok_vlook(MAIL)) == NULL)
+   if (!force && ibox != NULL) {
+      /* Folder extra introduced to avoid % recursion loops */
+      if ((cp = fexpand(ibox, FEXP_FOLDER | FEXP_NSHELL)) != NULL)
+         goto jcopy;
+      n_err(_("Failed to expand *ibox*, using $MAIL or default: %s\n"), ibox);
+      ibox = NULL;
+   }
+
+   if (force || (((cp = ibox) == NULL || *cp == '\0') &&
+         (cp = ok_vlook(MAIL)) == NULL))
       snprintf(buf, bufsize, "%s/%s", MAILSPOOL, user);
    else
 jcopy:
       n_strlcpy(buf, cp, bufsize);
-jleave:
    NYD_LEAVE;
 }
 
@@ -997,7 +1002,8 @@ setptr(FILE *ibuf, off_t offset)
    struct message self;
    char *cp, *linebuf = NULL;
    char const *cp2;
-   int c, maybe = 1, inhead = 0, selfcnt = 0;
+   int c, selfcnt = 0;
+   bool_t maybe, inhead, rfc4155;
    size_t linesize = 0, filesize, cnt;
    NYD_ENTER;
 
@@ -1005,6 +1011,8 @@ setptr(FILE *ibuf, off_t offset)
    self.m_flag = MUSED | MNEW | MNEWEST;
    filesize = mailsize - offset;
    offset = ftell(mb.mb_otf);
+   maybe = TRU1;
+   rfc4155 = inhead = FAL0;
 
    for (;;) {
       if (fgetline(&linebuf, &linesize, &filesize, &cnt, ibuf, 0) == NULL) {
@@ -1034,10 +1042,19 @@ setptr(FILE *ibuf, off_t offset)
       }
       if (linebuf[cnt - 1] == '\n')
          linebuf[cnt - 1] = '\0';
-      if (maybe && linebuf[0] == 'F' && is_head(linebuf, cnt, FAL0)) {
+      if (maybe && linebuf[0] == 'F' &&
+            (rfc4155 = is_head(linebuf, cnt, TRU1))) {
          /* TODO char date[FROM_DATEBUF];
           * TODO extract_date_from_from_(linebuf, cnt, date);
           * TODO self.m_time = 10000; */
+         if (rfc4155 == TRUM1) {
+            if (options & OPT_D_V)
+               n_err(_("Invalid MBOX \"From_ line\": %.*s\n"),
+                  (int)cnt, linebuf);
+            else if (!(mb.mb_active & MB_FROM__WARNED))
+               n_err(_("MBOX mailbox contains non-conforming From_ line(s)\n"));
+            mb.mb_active |= MB_FROM__WARNED;
+         }
          self.m_xsize = self.m_size;
          self.m_xlines = self.m_lines;
          self.m_have = HAVE_HEADER | HAVE_BODY;
@@ -1049,9 +1066,9 @@ setptr(FILE *ibuf, off_t offset)
          self.m_lines = 0;
          self.m_block = mailx_blockof(offset);
          self.m_offset = mailx_offsetof(offset);
-         inhead = 1;
+         inhead = TRU1;
       } else if (linebuf[0] == 0) {
-         inhead = 0;
+         inhead = FAL0;
       } else if (inhead) {
          for (cp = linebuf, cp2 = "status";; ++cp) {
             if ((c = *cp2++) == 0) {
@@ -1091,7 +1108,7 @@ setptr(FILE *ibuf, off_t offset)
       offset += cnt;
       self.m_size += cnt;
       ++self.m_lines;
-      maybe = linebuf[0] == 0;
+      maybe = (linebuf[0] == 0);
    }
    NYD_LEAVE;
 }
@@ -1133,7 +1150,7 @@ setinput(struct mailbox *mp, struct message *m, enum needspec need)
 
    fflush(mp->mb_otf);
    if (fseek(mp->mb_itf, (long)mailx_positionof(m->m_block, m->m_offset),
-         SEEK_SET) < 0) {
+         SEEK_SET) == -1) {
       n_perr(_("fseek"), 0);
       n_panic(_("temporary file seek"));
    }
@@ -1260,7 +1277,7 @@ fsize(FILE *iob)
    off_t rv;
    NYD_ENTER;
 
-   rv = (fstat(fileno(iob), &sbuf) < 0) ? 0 : sbuf.st_size;
+   rv = (fstat(fileno(iob), &sbuf) == -1) ? 0 : sbuf.st_size;
    NYD_LEAVE;
    return rv;
 }
@@ -1281,10 +1298,13 @@ fexpand(char const *name, enum fexp_mode fexpm)
    if ((fexpm & FEXP_NSHORTCUT) || (res = shortcut_expand(name)) == NULL)
       res = UNCONST(name);
 
-   if (fexpm & FEXP_SHELL) {
+   if (fexpm & (FEXP_SHELL | FEXP_FOLDER)) {
       dyn = FAL0;
-      goto jshell;
+      if (fexpm & FEXP_SHELL)
+         goto jshell;
+      goto jfolder;
    }
+
 jnext:
    dyn = FAL0;
    switch (*res) {
@@ -1322,11 +1342,9 @@ jnext:
       dyn = TRU1;
    }
 
+jfolder:
    if (res[0] == '+' && getfold(cbuf, sizeof cbuf)) {
-      size_t i = strlen(cbuf);
-
-      res = str_concat_csvl(&s, cbuf,
-            ((i > 0 && cbuf[i - 1] == '/') ? "" : "/"), res + 1, NULL)->s;
+      res = str_concat_csvl(&s, cbuf, &res[1], NULL)->s;
       dyn = TRU1;
 
       if (res[0] == '%' && res[1] == ':') {
@@ -1341,7 +1359,7 @@ jshell:
       res = str_concat_csvl(&s, homedir, res + 1, NULL)->s;
       dyn = TRU1;
    }
-   if (anyof(res, "|&;<>{}()[]*?$`'\"\\"))
+   if (anyof(res, n_SHEXP_MAGIC_PATH_CHARS))
       switch (which_protocol(res)) {
       case PROTO_FILE:
       case PROTO_MAILDIR:
@@ -1470,8 +1488,14 @@ getfold(char *name, size_t size)
    char const *folder;
    NYD_ENTER;
 
-   if ((folder = ok_vlook(folder)) != NULL)
+   if ((folder = ok_vlook(folder)) != NULL){
+      size_t i;
+
+      i = strlen(folder);
+      if(i > 0 && folder[i - 1] != '/')
+         folder = savecat(folder, "/");
       n_strlcpy(name, folder, size);
+   }
    NYD_LEAVE;
    return (folder != NULL);
 }
