@@ -134,7 +134,7 @@ _cleantmp(void)
       if (dp->d_name[0] == '.')
          continue;
       sstpcpy(sstpcpy(dep, "tmp/"), dp->d_name);
-      if (stat(dep, &st) < 0)
+      if (stat(dep, &st) == -1)
          continue;
       if (st.st_atime + 36*3600 < now)
          unlink(dep);
@@ -312,7 +312,7 @@ readin(char const *name, struct message *m)
       /* Since we simply copy over data without doing any transfer
        * encoding reclassification/adjustment we *have* to perform
        * RFC 4155 compliant From_ quoting here */
-      if (emptyline && is_head(buf, buflen, TRU1)) {
+      if (emptyline && is_head(buf, buflen, FAL0)) {
          putc('>', mb.mb_otf);
          ++size;
       }
